@@ -25,14 +25,16 @@
 #include "driver.h"
 #include "vkontakte.h"
 #include "parser.h"
+#include "profile.h"
 #define XSL_URL DATADIR"/msa/drivers/vkontakte/make_url.xsl"
 
 #ifdef TEST
-#include "../../../test/CTF/src/ctf.h"
+#include "../test/CTF/src/ctf.h"
 CTF_TEST_DATA_FILE_NAME_DECL;
 #endif
 
 int counter = 0;
+int max_count = 0;
 static int global_temp = 0;
 int attempts = 0;
 int captch = 0;
@@ -106,7 +108,7 @@ xmlChar* event_message = xmlCharStrdup("\
     xmlDocDumpMemory(doc_root, &doc_char, &size);
     g_debug("set_event");
 
-    set_events(doc_char);
+    //set_events(doc_char);
 
     xmlXPathFreeObject(Obj);
     xmlFreeDoc(doc);
@@ -124,7 +126,7 @@ xmlChar* event_message = xmlCharStrdup("\
  * @param size - size of data
 * @return 0 on success or 1 otherwise.
  */
-static void* myrealloc(void* ptr, size_t size)
+void* myrealloc(void* ptr, size_t size)
 {
 /* There might be a realloc() out there that doesn't like reallocing
 NULL pointers, so we take care of it here */
@@ -188,15 +190,17 @@ static char* get_captcha(guint32 captcha_sid)
  * @param doc - response
  * @return 0 on success or 1 otherwise.
  */
-int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
+int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, const struct msa_module* info)
 {
     captch = 0;
     int init_status = 0;
-    gchar* remixmid = NULL;
-    gchar* remixsid = NULL;
     xmlChar* class_val = NULL;
     xmlChar* function = NULL;
-    xmlXPathObject *Obj = NULL;
+
+    struct ProfileData *cp = (struct ProfileData *)profile_get_by_id(info->id);
+    g_debug("%s:%d:%s: id=%s, cp->login=%s", __FILE__, __LINE__, __FUNCTION__, info->id, cp->login);
+    info_proxy = info->proxy;
+    info_port = info->port;
 
     global_temp++;
     
@@ -223,14 +227,39 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     childNode = xmlAddChild(rootNode, node);
  
     g_debug("function = %s", function);
+
+    xmlChar* temp = NULL;
+    if (g_strcmp0((gchar *)function, GET_LIST_FRIENDS) == 0 || 
+        g_strcmp0((gchar *)function, UPDATE_INBOX_MESSAGES) == 0 ||
+        g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0) {
+
+        node = xpath_get_node("//Request/Params/number[@name='from']", reqXml);
+        if(node != NULL){
+            g_debug("node not NULL");
+            temp = xmlNodeGetContent(node);
+            counter = atoi((gchar *)temp);
+            g_free(temp); 
+        } else 
+            counter = 0;
+
+        node = xpath_get_node("//Request/Params/number[@name='to']", reqXml);
+        if(node != NULL){
+            g_debug("node not NULL");
+            temp = xmlNodeGetContent(node);
+            max_count = atoi((gchar *)temp);
+            g_free(temp);
+        } else 
+            max_count = MAX_COUNTER;
+    }
+
     if (g_strcmp0((gchar *)function, GET_SETTINGS) == 0) {
-        get_settings(doc);
+        get_settings(doc, cp);
         xmlFree(function);
        return 0;
     }
     
     if(g_strcmp0((gchar *)function, SET_SETTINGS) == 0) {
-        if (set_settings(reqXml, doc, info)) {
+        if (set_settings(reqXml, cp)) {
             info_message("404", doc, DRV_SETT_ERROR, 1);
         } else {
             info_message("410", doc, DRV_SETT_SAVED, 0);
@@ -243,31 +272,28 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
         captch = 1;
 
         xmlChar* captcha_code = NULL;
-        Obj = xpath("//Request/Params/string[@name='text']", reqXml);
-        node = Obj->nodesetval->nodeTab[0];
+        node = xpath_get_node("//Request/Params/string[@name='text']", reqXml);
+        if(node == NULL)
+            return 1;
         captcha_code = xmlNodeGetContent(node);
         g_debug("captcha code %s", captcha_code);
-        xmlXPathFreeObject(Obj);
 
-        Obj = xpath("//Request/Params/string[@name='oldRequest']/Request/Params/string[@name='fccode']", reqXml);
-        node = Obj->nodesetval->nodeTab[0];
+        node = xpath_get_node("//Request/Params/string[@name='oldRequest']/Request/Params/string[@name='fccode']", reqXml);
+        if(node == NULL)
+            return 1;
         xmlNodeSetContent(node, captcha_code);
-        xmlXPathFreeObject(Obj);
 
-        Obj = xpath("//Request/Params/string[@name='oldRequest']/Request", reqXml);
-	
-	if(Obj != NULL){
-            node = Obj->nodesetval->nodeTab[0];
+        node = xpath_get_node("//Request/Params/string[@name='oldRequest']/Request", reqXml);
+	    if(node != NULL){
             childNode = xmlCopyNode(node,1);
             xmlUnlinkNode(childNode);
             reqXml2 = xmlNewDoc(BAD_CAST "1.0");
             xmlNodePtr oldChild = xmlDocSetRootElement(reqXml2, childNode);
             if (oldChild != NULL)
                 xmlFreeNode(oldChild);
-            xmlXPathFreeObject(Obj);
         } else {
             return 1;
-       }
+          }
     }
 
     if(captch) {
@@ -281,7 +307,7 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     if(function)
         xmlFree(function);
   
-    init_status = initialization(&remixmid, &remixsid);
+    init_status = initialization(cp);
     attempts = 0;
     if (init_status != 0) {
         if(init_status == -2) {
@@ -304,14 +330,10 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
     }
 
     if(captch)
-        make_request( reqXml2, doc, remixmid, remixsid);
+        make_request(reqXml2, doc, cp);
     else
-        make_request( reqXml, doc, remixmid, remixsid);
+        make_request(reqXml, doc, cp);
 
-    if(remixmid)
-        g_free(remixmid);
-    if(remixsid)
-        g_free(remixsid);
     return 0;
 }
 
@@ -320,22 +342,20 @@ int vk_req(xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
  *
  * @param reqXml - request
  * @param doc - response
- * @param remixmid - authorization parameter
- * @param remixsid - authorization parameter
+ * @param cp - current profile
 * @return 0 on success or 1 otherwise.
  */
-int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid)
+int make_request(xmlDocPtr reqXml, xmlDocPtr doc, struct ProfileData *cp)
 {
     gchar* converted_friends = NULL;
-    char* tmp = NULL;
+    gchar* tmp = NULL;
     char* url;
     //gchar* class = get_req_class(reqXml);
     xmlChar* function = get_req_function(reqXml);
     int num = 0;
     int init_status = 0;
-    xmlXPathObject *Obj = NULL;
     
-    url = make_url(reqXml, remixmid, remixsid);
+    url = make_url(reqXml, cp->remixmid, cp->remixsid);
     
     if (url == NULL || strlen(url) < 1) {
         if (g_strcmp0((gchar *)function, GET_BINARY_DATA) == 0 ) {
@@ -361,7 +381,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
             g_free(url);
             g_free(tmp);
             counter++;
-            make_request(reqXml, doc, remixmid, remixsid);
+            make_request(reqXml, doc, cp);
             return 0;
         }
         g_free(url);
@@ -383,11 +403,19 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
 
                 if (g_strcmp0((gchar *)function, GET_MY_PROFILE) == 0 || 
                     g_strcmp0((gchar *)function, UPDATE_PROFILE) == 0) {
-                    if(parser_profile(doc, tmp, remixmid) == 1){
+                    if(parser_profile(doc, tmp, cp->remixmid, TRUE) == 1){
                         info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
                     }
                     g_free(tmp);      	    
                 }
+                if (g_strcmp0((gchar *)function, GET_PROFILE_NO_IMAGE) == 0 ) {
+                    if(parser_profile(doc, tmp, cp->remixmid, FALSE) == 1){
+                        info_message(DRV_RESP_ERROR_CODE, doc, DRV_RESP_ERROR, 1);
+                    }
+                    g_free(tmp);      	    
+                }
+
+
                 if (g_strcmp0((gchar *)function, SEND_MY_PROFILE) == 0) {
                     info_message(DRV_PROF_SAVED_CODE, doc, DRV_PROF_SAVED, 0);
                     g_free(tmp);
@@ -395,13 +423,14 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                     return 0;
                 }
                 if (g_strcmp0((gchar *)function, GET_LIST_FRIENDS) == 0) {
+                    g_debug("GET_LIST_FRIENDS");
                     converted_friends = g_convert(tmp, -1, UTF8, CP1251, NULL, NULL, NULL);
                     g_free(tmp);
-                    num = parser_friends(converted_friends, doc);
+                    num = parser_friends(converted_friends, doc, counter);
                     g_free(converted_friends);
-                    if (num == PAGE_SIZE) {
-                        counter++;
-                        make_request( reqXml, doc, remixmid, remixsid);
+                    if (num == PAGE_SIZE && (counter + PAGE_SIZE) < max_count ) {
+                        counter += PAGE_SIZE;
+                        make_request( reqXml, doc, cp);
                     } else {
                         xmlFree(function);
                         return 0;
@@ -409,20 +438,18 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                 }
                 if (g_strcmp0((gchar *)function, UPDATE_INBOX_MESSAGES) == 0 || 
                     g_strcmp0((gchar *)function, UPDATE_OUTBOX_MESSAGES) == 0) {
-                    char* message_id = NULL;
-                    Obj = xpath("//Request/Params", reqXml);
-                    if(Obj != NULL) {
-                        xmlNodePtr node;
-                        node = Obj->nodesetval->nodeTab[0];
-                        message_id = xmlGetProp(node, "messageId");
-                        xmlXPathFreeObject(Obj);
-                    }
-                    num = parser_messages(tmp, doc, message_id);
+                    xmlChar* message_id = NULL;
+                    xmlNodePtr node;
+                    node = xpath_get_node("//Request/Params", reqXml);
+                    if(node != NULL)
+                        message_id = xmlGetProp(node, BAD_CAST "messageId");
+
+                    num = parser_messages(tmp, doc, (gchar *)message_id, counter, max_count);
                     g_free(tmp);
-                    if (num == PAGE_SIZE && counter < MAX_COUNTER) {
+                    if (num == PAGE_SIZE && (counter + PAGE_SIZE) < max_count) {
                         g_debug("num_messages = %d", num);
-                        counter++;
-                        make_request( reqXml, doc, remixmid, remixsid);
+                        counter += PAGE_SIZE;
+                        make_request( reqXml, doc, cp);
                     } else {
                         xmlFree(function);
                         return 0;
@@ -434,10 +461,11 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                     g_free(tmp);
                     g_debug("num = %d",num);
                     if (num > 0) {
-                        url = g_strdup_printf("http://userapi.com/data?act=inbox&from=0&to=%d&id=%s&sid=%s",num,remixmid,remixsid);
+                        url = g_strdup_printf("http://userapi.com/data?act=inbox&from=0&to=%d&id=%s&sid=%s",
+                                num, cp->remixmid, cp->remixsid);
                         if(http_get(url, info_proxy, info_port, NULL, &tmp) == 200){
                             g_free(url);
-                            num = parser_messages(tmp, doc, NULL);
+                            num = parser_messages(tmp, doc, NULL, 0, 0);
                             g_free(tmp);
 			    //read_message(doc, remixmid, remixsid);
                         } else {
@@ -476,12 +504,14 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                 }
                 break;
             case -1:
-                unset_settings(REMIXSID);
+                g_free(cp->remixsid);
+                cp->remixsid = NULL;
+
                 if (attempts == 0) {
                     attempts = 1;
-                    init_status = initialization(&remixmid, &remixsid);
+                    init_status = initialization(cp);
                     if (init_status == 0)
-                        make_request(reqXml, doc, remixmid, remixsid);
+                        make_request(reqXml, doc, cp);
                 }
                 else 		
                     info_message(DRV_AUTH_ERROR_CODE, doc, DRV_AUTH_ERROR, 1);
@@ -494,7 +524,7 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
                 g_free(function);
 		break;
             case -3:
-                g_debug(tmp);
+                g_debug("%s", tmp);
                 info_message(DRV_ACCESS_ERROR_CODE, doc, DRV_ACCESS_ERROR, 1);
                 g_free(function);
                 g_free(tmp);
@@ -518,152 +548,74 @@ int make_request(xmlDocPtr reqXml, xmlDocPtr doc, char* remixmid, char* remixsid
 /**
  * Perform authorization
  *
- * @param remixmid - authorization parameter
- * @param remixsid - authorization parameter
-* @return 0 on success or 1 otherwise.
+ * @param cp - current profile settings
+ * @return 0 on success or 1 otherwise.
  */
-int initialization(char** remixmid, char** remixsid)
+int initialization(struct ProfileData *cp)
 {
     g_debug("init START");
 
-    char* email;
-    char* pass;
     int auth_status = 0;
-    gchar* path;
-    GConfClient* client;
-    client = gconf_client_get_default ();
-#ifndef TEST
-    path = g_strconcat(PATH, REMIXMID, NULL);
-    *remixmid = gconf_client_get_string (client, path, NULL);
-    g_free(path);
-    
-    path = g_strconcat(PATH, REMIXSID, NULL);
-    *remixsid = gconf_client_get_string (client, path, NULL);
-    g_free(path);
-#else
-    email = CTF_TEST_DATA("LOGIN");
-    pass = CTF_TEST_DATA("PASSWORD");
-#endif
 
-    if(*remixmid == NULL || *remixsid == NULL ) {
-        path = g_strconcat(PATH, LOGIN, NULL);
-        email = gconf_client_get_string (client, path, NULL);
-        g_free(path);
-
-        path = g_strconcat(PATH, PASSWORD, NULL);
-        pass = gconf_client_get_string (client, path, NULL);
-        g_free(path);
+    if(cp->remixmid == NULL || cp->remixsid == NULL ) {
 
         g_print("dr_vk: authorization\n");
-        auth_status = authorize(email, pass);
-        g_free(email);
-        g_free(pass);
+        auth_status = authorize(cp);
         if (auth_status != 0 ) {
             g_debug("dr_vk: Could not get remixmid, remixsid\n"); 
             //if (auth_status == -2 ) {
             //    return -2;
             //}
             return auth_status;
-        } else {
-            path = g_strconcat(PATH, REMIXMID, NULL); 
-            *remixmid = gconf_client_get_string (client, path, NULL);
-            g_free(path);
-            path = g_strconcat(PATH, REMIXSID, NULL);
-            *remixsid = gconf_client_get_string (client, path, NULL);
-            g_free(path);
         }
     }
-    g_object_unref((gpointer)client);
     g_debug("init END");
     return 0;
 }
 
 /**
- * Get settings from GConf
+ * Get settings from current profile
  *
  * @param doc - response
  * @return 0 on success or 1 otherwise.
  */
-int get_settings(xmlDocPtr doc)
+int get_settings(xmlDocPtr doc, const struct ProfileData *cp)
 {
     g_debug("get_settings START");
 
-    GConfClient* client;
-    gchar* login;
-    gchar* password;
-    gchar* path;
-    gchar* tmp;
-    gint status = 0, autoupdate = 0;
     xmlNodePtr rootNode, childNode;
     
     rootNode = xmlDocGetRootElement(doc);
     rootNode = rootNode->children;
     
-    client = gconf_client_get_default ();
-
-    path = g_strconcat(PATH, STATUS, NULL);
-    status = gconf_client_get_int (client, path, NULL);
-    g_free(path);
-
-    path = g_strconcat(PATH, LOGIN, NULL);
-    login = gconf_client_get_string (client, path, NULL);
-    g_free(path);
-
-    path = g_strconcat(PATH, PASSWORD, NULL);
-    password = gconf_client_get_string (client, path, NULL);
-    g_free(path);
-
-    path = g_strconcat(PATH, AUTOUPDATE, NULL);
-    autoupdate = gconf_client_get_int (client, path , NULL);
-    g_free(path);
-    
-    childNode = xmlNewDocNode(doc, NULL, BAD_CAST STRING, BAD_CAST login);
+    childNode = xmlNewDocNode(doc, NULL, BAD_CAST STRING, BAD_CAST cp->login);
     xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST LOGIN);
     childNode = xmlAddChild(rootNode, childNode);
-    g_free(login);
 
-    childNode = xmlNewDocNode(doc, NULL, BAD_CAST STRING, BAD_CAST password);
+    childNode = xmlNewDocNode(doc, NULL, BAD_CAST STRING, BAD_CAST cp->password);
     xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST PASSWORD);
     childNode = xmlAddChild(rootNode, childNode);
-    g_free(password);
 
-    tmp = g_strdup_printf("%d", autoupdate);
-    childNode = xmlNewDocNode(doc, NULL, BAD_CAST NUMBER, BAD_CAST tmp);
-    xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST AUTOUPDATE);
-    childNode = xmlAddChild(rootNode, childNode);
-    g_free(tmp);
-    
-    tmp = g_strdup_printf("%d", status);
-    childNode = xmlNewDocNode(doc, NULL, BAD_CAST BOOLEAN, BAD_CAST tmp);
-    xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST STATUS);
-    childNode = xmlAddChild(rootNode, childNode);
-    g_free(tmp);
-
-    g_object_unref((gpointer)client);
     g_debug("get_settings END");
    return 0;
 }
 
 /**
- * Set settings in GConf 
+ * Set settings in to current profile
  *
  * @param reqXml - request
- * @param doc - response
-* @return 0 on success or 1 otherwise.
+ * @param cp - current profile
+ * @return 0 on success or 1 otherwise.
  */
-int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
+int set_settings (xmlDocPtr reqXml, struct ProfileData *cp)
 {
     g_debug("set_settings START");
 
-    GConfClient* client;
     gboolean checked = 1;
     int status = 0;
-    gchar* path;
-    xmlChar* temp;
     xmlChar* name;
     xmlNodePtr rootNode;
 
-    client = gconf_client_get_default ();
     rootNode = xmlDocGetRootElement(reqXml);
     rootNode = rootNode->children;
     rootNode = rootNode->children;
@@ -672,49 +624,23 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
         g_debug("%s", rootNode->name);
         name = xmlGetProp(rootNode, BAD_CAST NAME);
         if (g_strcmp0((gchar *)name, LOGIN) == 0) {
-            path = g_strconcat(PATH, LOGIN, NULL);
-            temp = xmlNodeGetContent(rootNode);	    
-            checked = gconf_client_set_string (client, path, (gchar *)temp, NULL);
-            g_free(path);    
-	    xmlFree(temp);
+            if (cp->login != NULL)
+            {
+                g_free(cp->login);
+                cp->login = NULL;
+            }
+            cp->login = (gchar *)xmlNodeGetContent(rootNode);
         }
         xmlFree(name);
         
         name = xmlGetProp(rootNode, BAD_CAST NAME);
         if (g_strcmp0((gchar *)name, PASSWORD) == 0) {
-            path = g_strconcat(PATH, PASSWORD, NULL);
-	    temp = xmlNodeGetContent(rootNode);
-            checked = gconf_client_set_string (client, path, (gchar *)temp, NULL);
-            g_free(path);
-	    xmlFree(temp);
-        }
-        xmlFree(name);
-        
-        name = xmlGetProp(rootNode, BAD_CAST NAME);
-        if (g_strcmp0((gchar *)name, AUTOUPDATE) == 0) {
-            int time = 0;
-	    temp = xmlNodeGetContent(rootNode);
-            time = atoi((gchar *)temp);
-            path = g_strconcat(PATH, AUTOUPDATE, NULL);
-            checked = gconf_client_set_int (client, path, time, NULL);
-#ifndef TEST
-	    init_event_manager(time);
-#endif   
-            g_free(path);
-	    xmlFree(temp);
-        }
-        xmlFree(name);
-        
-        name = xmlGetProp(rootNode, BAD_CAST NAME);
-        if (g_strcmp0((gchar *)name, STATUS) == 0) {
-            path = g_strconcat(PATH, STATUS, NULL);
-	    temp = xmlNodeGetContent(rootNode);
-#ifndef TEST
-            info->status = atoi((gchar *)temp);
-#endif
-            checked = gconf_client_set_int (client, path, atoi((gchar *)temp), NULL);
-            g_free(path);
-	    xmlFree(temp);
+            if (cp->password != NULL)
+            {
+                g_free(cp->password);
+                cp->password = NULL;
+            }
+	    cp->password = (gchar *)xmlNodeGetContent(rootNode);
         }
         xmlFree(name);
         
@@ -723,40 +649,30 @@ int set_settings (xmlDocPtr reqXml, xmlDocPtr doc, msa_module* info)
             status = 1;
     }
 
-    unset_settings(REMIXMID);
-    unset_settings(REMIXSID);
+    if (cp->remixmid != NULL)
+    {
+        g_free(cp->remixmid);
+        cp->remixmid = NULL;
+    }
+
+    if (cp->remixsid != NULL)
+    {
+        g_free(cp->remixsid);
+        cp->remixsid = NULL;
+    }
     
+    g_debug("%s:%d:%s: login=%s password=%s", __FILE__, __LINE__, __FUNCTION__, cp->login, cp->password);
+
     return status;
 }
 
 /**
- * Unset value in gconf
- * @param name - name of setting for unset
-*/
-void unset_settings(char* name){
-    GConfClient* client;
-    gchar* path;
-
-    client = gconf_client_get_default ();
-
-    path = g_strconcat(PATH, name, NULL);
-    gconf_client_unset(client, path, NULL);
-    g_free(path);
-
-    g_object_unref((gpointer)client);
-}
-
-/**
- * Authorization
+ * Authorization (get remixsid and remixmid for future requests)
  *
- * @param remixmid - auth parameter
- * @param remixsid - auth parameter
- * @param email - email
- * @param pass - password
- * @param cookies - file for cookies
+ * @param cp - current profile
  * @return 0 on success or 1 otherwise.
  */
-int authorize(char *email, char *pass)
+int authorize(struct ProfileData *cp)
 {
     char *body = NULL;
     char* mid = NULL, sid[MID_SIZE];
@@ -764,12 +680,10 @@ int authorize(char *email, char *pass)
     gchar* post_data;
     int code = 0;
     struct curl_slist *cookies = NULL;
-    gchar* path;
 
-    GConfClient* client;
-    client = gconf_client_get_default ();
-    if ( email == NULL || pass == NULL) {
-        g_debug("email = NULL");
+
+    if ( cp->login == NULL || cp->password == NULL) {
+        g_debug("login or password = NULL");
         return 1;
     }
 
@@ -779,7 +693,7 @@ int authorize(char *email, char *pass)
 #endif
 
     g_debug("%s:%d:%s(): Get mid", __FILE__, __LINE__, __FUNCTION__);
-    post_data = g_strdup_printf("email=%s&pass=%s&expire=&vk=1&try_to_login=1", email, pass);
+    post_data = g_strdup_printf("email=%s&pass=%s&expire=&vk=1&try_to_login=1", cp->login, cp->password);
 
     code = http_post("http://login.vk.com/?act=login", 
 		     info_proxy, info_port, post_data, NULL, &body, &cookies);
@@ -807,10 +721,7 @@ int authorize(char *email, char *pass)
 
         curl_slist_free_all(cookies_orig);
 	if ( mid != NULL ) {
-	    path = g_strconcat(PATH, REMIXMID, NULL);
-	    gconf_client_set_string (client, path, mid, NULL);
-	    g_free(path);
-            g_free(mid);
+            cp->remixmid = mid;
 	} else {
             g_debug("mid not found");
 	    //g_free(mid);
@@ -827,26 +738,28 @@ int authorize(char *email, char *pass)
             }
             
             if(tmp == NULL){
-                path = g_strconcat(PATH, REMIXMID, NULL);
-                gconf_client_unset(client, path, NULL);
+                if (cp->remixmid != NULL) {
+                    g_free(cp->remixmid);
+                    cp->remixmid = NULL;
+                }
                 g_free(body);
-                g_free(path);
 	        g_debug("no sid");
 	        return 1;
             } else {
 	        g_debug("sid found");
 	        sscanf(tmp, "%s", sid);
 	        sid[strlen(sid)-1]='\0';
-                path = g_strconcat(PATH, REMIXSID, NULL);
-                gconf_client_set_string (client, path, sid, NULL);
-                g_free(path);
+                if (cp->remixsid != NULL)
+                    g_free(cp->remixsid);
+                cp->remixsid = g_strdup(sid);
                 g_free(body);
                 return 0;
             }
         }
-	path = g_strconcat(PATH, REMIXMID, NULL);
-	gconf_client_unset(client, path, NULL);
-	g_free(path);
+        if (cp->remixmid != NULL) {
+            g_free(cp->remixmid);
+            cp->remixmid = NULL;
+        }
         return 1;
     } else {
         curl_slist_free_all(cookies);
@@ -857,91 +770,6 @@ int authorize(char *email, char *pass)
     }
 }
 
-
-/*
-int authorize(char *email, char *pass)
-{
-    g_type_init();
-    FILE* fpin;
-    int i;
-    char *buf;
-    char url[URL_SIZE];
-    char mid[MID_SIZE], sid[MID_SIZE];
-    char *tmp;
-    char *resp;
-    int code = 0;
-    gchar* path;
-    
-    g_print("authorize START");
-     GConfClient* client;
-     client = gconf_client_get_default ();
-    if ( email == NULL || pass == NULL) {
-        g_debug("email = NULL");
-        return 1;
-    }
-    //sprintf(url, "http://vkontakte.ru/login.php?email=%s&pass=%s", email, pass);
-    sprintf(url, "http://login.userapi.com/auth?login=force&site=2&email=%s&pass=%s", email, pass);
-    g_debug(" %s \n",url); // наш запрос
-#ifdef TEST
-    info_proxy = g_strdup("proxy.karelia.ru");
-    info_port = 81;
-#endif
-    code = http_get(url, info_proxy, info_port, &buf, NULL); 
-    if (code == 302) {
-        g_debug("HEADER\n  %s", buf);
-    
-        if (buf != NULL) {
-            tmp = NULL;
-            tmp = g_strrstr(buf, "sid=") + 4;
-            if(tmp == NULL) {
-                g_debug("no sid");
-                g_free(buf);
-                return 1;
-            } else
-                g_debug("sid found");
-            sscanf(tmp, "%s", sid);
-            sid[strlen(sid)] = '\0';
-            if ( sid != NULL ) {
-                path = g_strconcat(PATH, REMIXSID, NULL);    
-                gconf_client_set_string (client, path, sid, NULL);
-		g_free(path);
-            } else {
-                g_free(buf);
-                return 1;
-            }
-            tmp = NULL;
-            tmp = g_strrstr(buf, "remixmid=");
-            if(tmp == NULL) {
-                g_debug("no remixmid");
-                path = g_strconcat(PATH, REMIXSID, NULL);
-                gconf_client_unset(client, path, NULL);
-                g_free(path);
-                g_free(buf);
-                return 1;
-            } else 
-                tmp = tmp + 9;
-            sscanf(tmp, "%s", mid);
-            mid[strlen(mid)-1] = '\0';
-            if( mid != NULL ) {
-		path = g_strconcat(PATH, REMIXMID, NULL);
-                gconf_client_set_string (client, path, mid, NULL);
-		g_free(path);
-                g_free(buf);
-            } else {
-                g_free(buf);
-                return 1;
-            }
-        } else 
-            return 1;
-    }
-    else if (code == 200)
-        return 1;
-    else
-        return 1;
-   return 0;
-}
-*/
-
 /**
  * Get class of request
  *
@@ -1017,7 +845,7 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
 {
     g_debug("make_url START");
     xmlChar* url;
-    const char* params[9];
+    const char* params[11];
     xmlDocPtr urlXml;
     xmlNodePtr root;
     //xmlDocDump(stdout, reqXml);
@@ -1028,12 +856,14 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     params[3] = g_strdup_printf("\"%s\"", remixsid);
     params[4] = g_strdup("COUNTER");
     params[5] = g_strdup_printf("%d", counter);
+    params[6] = g_strdup("MAXCOUNTER");
+    params[7] = g_strdup_printf("%d", max_count);
     if(captch == 1) {
-        params[6] = g_strdup("CAPTCHA");
-        params[7] = g_strdup("1");
-        params[8] = NULL;
+        params[8] = g_strdup("CAPTCHA");
+        params[9] = g_strdup("1");
+        params[10] = NULL;
     } else 
-        params[6] = NULL;
+        params[8] = NULL;
 
     xsltStylesheetPtr cur = NULL;
 
@@ -1062,9 +892,12 @@ gchar* make_url(xmlDocPtr reqXml, char* remixmid, char* remixsid)
     g_free((gpointer*)params[3]);
     g_free((gpointer*)params[4]);
     g_free((gpointer*)params[5]);
+    g_free((gpointer*)params[6]);
+    g_free((gpointer*)params[7]);
+
     if(captch == 1) {
-        g_free((gpointer*)params[6]);
-        g_free((gpointer*)params[7]);
+        g_free((gpointer*)params[8]);
+        g_free((gpointer*)params[9]);
         captch = 0;
     }
     g_printf("%s", url);
@@ -1127,9 +960,8 @@ int captcha_message(xmlDocPtr doc, xmlDocPtr reqXml)
     info_message("403", doc, DRV_CAPT_ERROR, 2);
 
     xmlNodePtr node_param, node, rootNode;
-    xmlXPathObject *Obj;
-    Obj = xpath("//Response/Params", doc);
-    node_param = Obj->nodesetval->nodeTab[0];
+
+    node_param = xpath_get_node("//Response/Params", doc);
 
     node = xmlNewDocNode(doc, NULL, BAD_CAST IMG, BAD_CAST captcha_image);
     xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Img");
@@ -1144,17 +976,15 @@ int captcha_message(xmlDocPtr doc, xmlDocPtr reqXml)
 
     xmlAddChild(node, rootNode);
 
-    Obj = xpath("//Response/Params/string[@name='oldRequest']/Request/Params", doc);
-    node_param = Obj->nodesetval->nodeTab[0];
+    node_param = xpath_get_node("//Response/Params/string[@name='oldRequest']/Request/Params", doc);
 
     if (node_param == NULL)
         g_debug("node_param == NULL");
     else {
 	gchar *str_sid = g_strdup_printf("%d", cpatcha_sid);
 
-        Obj = xpath("//Response/Params/string[@name='oldRequest']/Request/Params/string[@name='fcsid']", doc);
-        if (Obj->nodesetval->nodeNr != 0) {
-            node = Obj->nodesetval->nodeTab[0];
+    node = xpath_get_node("//Response/Params/string[@name='oldRequest']/Request/Params/string[@name='fcsid']", doc);
+        if (node != NULL) {
             xmlNodeSetContent(node, BAD_CAST str_sid);
         } else {
             node = xmlNewDocNode(doc, NULL, BAD_CAST STRING, BAD_CAST str_sid);
@@ -1182,10 +1012,10 @@ int captcha_message(xmlDocPtr doc, xmlDocPtr reqXml)
  * @param body - buffer for body of response
  * @return http server response code 
  */
-int http_get(const char* url, const char* proxy, long port, char** header, gchar** body)
+int http_get(const char* url, const char* proxy, int port, char** header, gchar** body)
 {
     CURL* c;
-    CURLcode code;
+    long code;
     char* type2;
     struct MemoryStruct chunk, chunk2;
     
@@ -1196,7 +1026,7 @@ int http_get(const char* url, const char* proxy, long port, char** header, gchar
     chunk2.size = 0;
 
     c = curl_easy_init();
-    g_print("url in http_get %s", url);
+    g_debug("%s:%d:%s:url in http_get %s, proxy=%s, port=%d", __FILE__, __LINE__, __FUNCTION__, url, proxy, port);
 
     curl_easy_setopt(c, CURLOPT_URL, url);
 
@@ -1217,12 +1047,11 @@ int http_get(const char* url, const char* proxy, long port, char** header, gchar
         return 400;
     }
 
-
-    curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
-    g_debug("CODE %d", code);
+    CURLcode ret = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
+    g_debug("CODE %ld", code);
 
     //type2 = g_malloc(sizeof(char)*50);
-    curl_easy_getinfo(c, CURLINFO_CONTENT_TYPE, &type2 );
+    ret = curl_easy_getinfo(c, CURLINFO_CONTENT_TYPE, &type2 );
     g_debug("TYPE %s", type2);
 
     if (header != NULL) {
@@ -1264,10 +1093,10 @@ int http_get(const char* url, const char* proxy, long port, char** header, gchar
  * @return http server response code
  */
 
-int http_post(const char* url, const char* proxy, long port, char* post_data, char** header, char** body, struct curl_slist** cookies)
+int http_post(const char* url, const char* proxy, int port, char* post_data, char** header, char** body, struct curl_slist** cookies)
 {
     CURL* c;
-    CURLcode code;
+    long code;
     CURLcode res;
     struct MemoryStruct chunk,chunk2;
 
@@ -1279,7 +1108,7 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
 
 
     c = curl_easy_init();
-    g_debug("url in http_post %s", url);
+    g_debug("%s:%d:%s:url in http_post %s, proxy=%s:%d", __FILE__, __LINE__, __FUNCTION__, url, proxy, port);
 
     curl_easy_setopt(c, CURLOPT_URL, url);
 
@@ -1290,7 +1119,7 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
 
     curl_easy_setopt(c, CURLOPT_POST, 1);
     curl_easy_setopt(c, CURLOPT_POSTFIELDS, post_data);
-    curl_easy_setopt(c, CURLOPT_POSTFIELDSIZE, -1);
+    curl_easy_setopt(c, CURLOPT_POSTFIELDSIZE, strlen(post_data));
     curl_easy_setopt(c, CURLOPT_FOLLOWLOCATION, 1);
 
     curl_easy_setopt(c, CURLOPT_COOKIEFILE, "");
@@ -1302,8 +1131,9 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
     curl_easy_setopt(c, CURLOPT_WRITEHEADER, (void *)&chunk2);
     g_debug("%s:%d:%s():send request", __FILE__, __LINE__, __FUNCTION__);
     
-    if (curl_easy_perform(c) != 0) {
-        g_warning("Can't perform the request\n");
+    CURLcode errornum = curl_easy_perform(c);
+    if (errornum != 0) {
+        g_warning("%s:%d:%s: Can't perform the request: %s", __FILE__, __LINE__, __FUNCTION__, curl_easy_strerror(errornum));
         curl_easy_cleanup(c);
         return 400;
     }
@@ -1316,8 +1146,8 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
         return 400;
     }
     
-    curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
-    g_debug("CODE %d", code);
+    res = curl_easy_getinfo(c, CURLINFO_RESPONSE_CODE, &code );
+    g_debug("CODE %ld", code);
     g_debug("HEADER\n  %s", chunk2.memory);
     //g_debug("BODY====\n  %s\n=====BODY_END======", chunk.memory);
     
@@ -1340,13 +1170,25 @@ int http_post(const char* url, const char* proxy, long port, char* post_data, ch
 }
 
 
+xmlNodePtr xpath_get_node(char* path, xmlDocPtr doc)
+{
+    xmlXPathObject *Obj = NULL;
+    xmlNodePtr node = NULL;
+    Obj = xpath(path, doc);
+    if(Obj == NULL)
+        return NULL;
+    if(!xmlXPathNodeSetIsEmpty(Obj->nodesetval))
+        node = xmlXPathNodeSetItem(Obj->nodesetval, 0);
+    xmlXPathFreeObject(Obj);
+    return node;
+}
 
 /**
  * @param req - xpath req
  * @param doc - xmlDocPtr  
  * @return xmlXPathObject*  
 **/
-xmlXPathObject* xpath(char* req, xmlDocPtr doc)
+xmlXPathObject* xpath(gchar* req, xmlDocPtr doc)
 {
 	xmlXPathObject *Obj;
 	xmlXPathContextPtr Ctx;
@@ -1360,7 +1202,7 @@ xmlXPathObject* xpath(char* req, xmlDocPtr doc)
 		return NULL;	
 	}
 
-	Obj = xmlXPathEvalExpression(req, Ctx);
+	Obj = xmlXPathEvalExpression(BAD_CAST req, Ctx);
 
 	g_debug("vk_xpath: end");
 
@@ -1369,3 +1211,4 @@ xmlXPathObject* xpath(char* req, xmlDocPtr doc)
 	return Obj;
 
 }
+
