@@ -120,30 +120,29 @@ int parser_image(xmlDocPtr doc, char* data, char* url)
  * @param json - data in json format
  * @return 0 on success or 1 otherwise.
  */
-int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
+int parser_profile(xmlDocPtr doc, char* json, char* remixmid, gboolean image)
 {
     int i = 0, buf_serialized = 0;
     gchar* id = NULL;
-    struct json_object* buf, *profile, *ht, *edu,*edu_item;
+    struct json_object* buf = NULL, *profile = NULL, *ht = NULL, *edu = NULL, *edu_item = NULL, *activity = NULL;
     
     xmlNodePtr rootNode, childNode, node, node2;
 
-    rootNode = xmlDocGetRootElement(doc);     //response 
-    rootNode = rootNode->children;           //params
-	
+    rootNode = xpath_get_node("//Response/Params", doc);
+	if(rootNode == NULL)
+        return 1;
+
     profile = json_tokener_parse(json);
     if(profile != NULL){
         buf = json_object_object_get(profile, ID_KEY );
         if(buf == NULL)
             return 1;
         //PARSE FRIEND'S ID
-        if (strcmp(remixmid, json_object_get_string(buf)) != 0 ) {
+        if (strcmp(remixmid, json_object_get_string(buf)) != 0){
             id = g_strconcat(PREFIX, json_object_get_string(buf), NULL);
             xmlSetProp(rootNode, BAD_CAST "id", BAD_CAST id);
             g_free(id);
-        }
-        //json_object_put(buf);
-	
+	    }
         //PARSE FIRST NAME
         buf = json_object_object_get(profile, FIRST_NAME_KEY );
         if(buf == NULL)
@@ -151,32 +150,47 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST FIRST_NAME);
         node = xmlAddChild(rootNode, node);
-        //json_object_put(buf);
 
-	//PARSE LAST NAME
+        //PARSE LAST NAME
         buf = json_object_object_get(profile, LAST_NAME_KEY );
         if(buf == NULL)
             return 1;
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST LAST_NAME);
         node = xmlAddChild(rootNode, node);
-        //json_object_put(buf);
-	
+
+        //PARSE ACTIVITY
+        buf = json_object_object_get(profile, ACTIVITY_KEY );
+        if(buf != NULL){
+            activity = json_object_object_get(buf, "5" );
+            if(activity != NULL){
+                node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(activity));
+                xmlSetProp(node, BAD_CAST NAME, BAD_CAST ACTIVITY);
+                node = xmlAddChild(rootNode, node);
+            }
+        }
         //PARSE IMAGE
         buf = json_object_object_get(profile, IMG_KEY );
         if(buf == NULL)
             return 1;
         char* img;
-        if(http_get(json_object_get_string(buf), info_proxy, info_port, NULL, &img) == 200) {
-            node = xmlNewDocNode(doc, NULL,  BAD_CAST "img", BAD_CAST img);
+        if (image){
+            if(http_get(json_object_get_string(buf), info_proxy, info_port, NULL, &img) == 200) {
+                node = xmlNewDocNode(doc, NULL,  BAD_CAST "img", BAD_CAST img);
+                xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Img");
+                xmlSetProp(node, BAD_CAST "uri", BAD_CAST json_object_get_string(buf));
+                node = xmlAddChild(rootNode, node);
+                g_debug("set image");
+	        free(img);
+            }
+        } else {
+            node = xmlNewDocNode(doc, NULL,  BAD_CAST "img", NULL);
             xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Img");
             xmlSetProp(node, BAD_CAST "uri", BAD_CAST json_object_get_string(buf));
             node = xmlAddChild(rootNode, node);
             g_debug("set image");
-	    free(img);
         }
-        //json_object_put(buf);
-        g_debug("free image buffer");
+
 
         ht = json_object_object_get(profile, HABITATION_KEY);
         if(ht == NULL)
@@ -190,13 +204,11 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST "CityId");
         node = xmlAddChild(childNode, node);
-        //json_object_put(buf);
 	
         buf = json_object_object_get(ht, CITY_KEY);
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST CITY_NAME);
         node = xmlAddChild(childNode, node);
-        //json_object_put(buf);
 	
         //PARSE COUNTRY
         childNode = xmlNewDocNode(doc, NULL,  BAD_CAST TYPE_STRUCT, NULL );
@@ -207,15 +219,12 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST "CountryId");
         node = xmlAddChild(childNode, node);
-        //json_object_put(buf);
 	
         buf = json_object_object_get(ht, COUNTRY_KEY);
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf) );
         xmlSetProp(node, BAD_CAST NAME, BAD_CAST COUNTRY_NAME);
         node = xmlAddChild(childNode, node);
-        //json_object_put(buf);
-        //json_object_put(ht);
-        g_debug("free ht");
+
 
         struct tm tm_date;
         memset(&tm_date, 0, sizeof(struct tm));
@@ -223,17 +232,14 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
         buf = json_object_object_get(profile, BIRTH_DAY_KEY );
         g_debug("%s:%d: day=%s", __FILE__, __LINE__, json_object_get_string(buf));
         tm_date.tm_mday = json_object_get_int(buf);
-        //json_object_put(buf);
 
         buf = json_object_object_get(profile, BIRTH_MONTH_KEY);
         g_debug("%s:%d: month=%s", __FILE__, __LINE__, json_object_get_string(buf));
         tm_date.tm_mon = json_object_get_int(buf) - 1;
-        //json_object_put(buf);
 	
         buf = json_object_object_get(profile, BIRTH_YEAR_KEY);
         g_debug("%s:%d: year=%s", __FILE__, __LINE__, json_object_get_string(buf));
         tm_date.tm_year = json_object_get_int(buf) - 1900;
-        //json_object_put(buf);
 	
         char date[20] = {0};
         if (tm_date.tm_year > 0 && tm_date.tm_mon > -1 &&  tm_date.tm_mday > 0) {
@@ -268,7 +274,6 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
 
         buf = json_object_object_get(profile, MALE_KEY );
         int sx_serialized = json_object_get_int(buf);
-        //json_object_put(buf);
 	
         for (i = 0; i < 3; i++) {
             node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST gender[i]);
@@ -305,7 +310,6 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
 
         buf = json_object_object_get(profile, POL_VIEW_KEY );
         int pol_view_serialized = json_object_get_int(buf);
-        //json_object_put(buf);
 	
         for (i = 0; i < 9; i++) {
             node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST pol_view[i]);
@@ -332,11 +336,11 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
        
                 buf = json_object_array_get_idx(edu_item, 2 );
                 buf_serialized = json_object_get_int(buf);
-                //json_object_put(buf);
                 if (buf_serialized < 21 || buf_serialized == 100 ) {
                     node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRUCT, NULL );
                     xmlSetProp(node, BAD_CAST NAME, BAD_CAST "Education");
                     node = xmlAddChild(childNode, node);
+                    node2 = NULL;
                     if (buf_serialized < 21) {
                         node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST "School" );
                         node2 = xmlAddChild(node, node2);
@@ -352,19 +356,15 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
                     node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
                     node2 = xmlAddChild(node, node2);
                     xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "Name");
-                    //json_object_put(buf);
 	
                     buf = json_object_array_get_idx(edu_item, 4 );
                     node2 = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_STRING, BAD_CAST json_object_get_string(buf));
                     node2 = xmlAddChild(node, node2);
                     xmlSetProp(node2, BAD_CAST NAME, BAD_CAST "DateComplete");
-                    //json_object_put(buf);
 	        }
-		//json_object_put(edu_item);
-		g_debug("free edu_item");
+
             }
-	    //json_object_put(edu);
-	    g_debug("free edu");
+
 	    
 	}
         json_object_put(profile);
@@ -382,33 +382,41 @@ int parser_profile(xmlDocPtr doc, char* json, char* remixmid)
  * @param json - data in json format
  * @return number of friends
  */
-int parser_friends(char *json, xmlDocPtr doc)
+int parser_friends(char *json, xmlDocPtr doc, int counter)
 {
     int i=0;
     int length = 0;
-    struct json_object *friend, *friends, *id, *name, *img, *status;
+    int friend_count = 0;
+    struct json_object *friend, *friends, *id, *name, *img, *status, *n, *fr, *container;
     char** friend_name;
     char* friend_surname;
     char* tmp;
     xmlNodePtr rootNode, childNode, node,node2;
 
-    rootNode = xmlDocGetRootElement(doc);     //response
-    if(rootNode == NULL)
-        return 0;   
-    rootNode = rootNode->children;           //params
-    if(rootNode == NULL)
+    rootNode = xpath_get_node("//Response/Params", doc);
+	if(rootNode == NULL)
         return 0;
-    childNode = rootNode->children;          //array
+    childNode = xpath_get_node("//Response/Params/array", doc);
 
     if (childNode == NULL) {
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
         childNode = xmlAddChild(rootNode, node);
         xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "contactList");
-        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST "0");
-        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST "0" );
+        tmp = g_strdup_printf("%d", counter);
+        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp);
+        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp);
+        g_free(tmp);
     }
 
-    friends = json_tokener_parse(json);
+    container = json_tokener_parse(json);
+    fr = json_object_object_get(container,"friends");
+    n = json_object_object_get(fr,"n");
+    friend_count = json_object_get_int(n);
+    tmp = g_strdup_printf("%d", friend_count);
+    xmlSetProp(childNode, BAD_CAST "quantity", BAD_CAST tmp);
+    g_free(tmp);
+
+    friends = json_object_object_get(fr,"d");
     length = json_object_array_length(friends);
     for(i = 0; i < length; i++) {
         friend = json_object_array_get_idx(friends, i );
@@ -500,34 +508,44 @@ int parser_friends(char *json, xmlDocPtr doc)
  * @param json - data in json format
  * @return number of messages
  */
-int parser_messages(char *json, xmlDocPtr doc)
+int parser_messages(char *json, xmlDocPtr doc, gchar *message_id, int counter, int max_count)
 {
     g_debug("parser_messages START");
     //g_debug("Messages JSON: \n %s", json);
     int i = 0;
     int length = 0;
-    struct json_object *mess, *message, *messages, *id, *sender, *time, *text, *status;
+    int message_count;
+    struct json_object *mess, *message, *messages, *id, *sender, *time, *text, *status, *n;
     char* tmp_str; 
     xmlNodePtr rootNode = NULL, childNode = NULL, node = NULL, node2 = NULL;
 
-    rootNode = xmlDocGetRootElement(doc);     //response 
-    rootNode = rootNode->children;            //params
-    childNode = rootNode->children;           //array
+    rootNode = xpath_get_node("//Response/Params", doc);
+	if(rootNode == NULL)
+        return 0;
+    childNode = xpath_get_node("//Response/Params/array", doc);
 
     if (childNode == NULL) {
         node = xmlNewDocNode(doc, NULL, BAD_CAST TYPE_ARRAY,  NULL);
         childNode = xmlAddChild(rootNode, node);
         xmlSetProp(childNode, BAD_CAST NAME, BAD_CAST "messagetList");
-        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST "0");
-        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST "0" );
+        tmp_str = g_strdup_printf("%d", counter);
+        xmlSetProp(childNode, BAD_CAST "from", BAD_CAST tmp_str);
+        xmlSetProp(childNode, BAD_CAST "to", BAD_CAST tmp_str);
+        g_free(tmp_str);
     }
 
     mess = json_tokener_parse(json);
+    n = json_object_object_get(mess,"n");
+    message_count = json_object_get_int(n);
+    tmp_str = g_strdup_printf("%d", message_count);
+    xmlSetProp(childNode, BAD_CAST "quantity", BAD_CAST tmp_str);
+    g_free(tmp_str);
+
     messages = json_object_object_get(mess,"d");
     length = json_object_array_length(messages);
     g_debug("Start parse");
-    if (length > PAGE_SIZE)
-        length = PAGE_SIZE;
+    if (length > (max_count - counter) || length > PAGE_SIZE)
+        length--;
     for(i = 0; i < length; i++) {
 
         message = json_object_array_get_idx(messages, i );
@@ -545,10 +563,12 @@ int parser_messages(char *json, xmlDocPtr doc)
         if(id == NULL){
             break;
         }
+        if(g_strcmp0(json_object_get_string(id), message_id) == 0)
+            break;
         tmp_str = g_strconcat(PREFIX, json_object_get_string(id), NULL);
         xmlSetProp(node, BAD_CAST "id", BAD_CAST tmp_str);
         node = xmlAddChild(childNode, node);
-	g_free(tmp_str);
+	    g_free(tmp_str);
 
         sender = json_object_array_get_idx(message, 3);
         if(sender == NULL)
@@ -623,9 +643,8 @@ int parser_messages(char *json, xmlDocPtr doc)
         text = json_object_array_get_idx(message, 2);
         if(text == NULL)
             break;
-        char* mess_text;
+        char* mess_text = json_object_get_string(text);
         char* tmp;
-        mess_text = json_object_get_string(text);
         while((tmp = strstr(mess_text, "<br>")) != NULL)
         {
             int k = 0;
@@ -698,4 +717,3 @@ int parser_new_messages(char* json)
     }
     return 0;
 }
-   
