@@ -40,6 +40,7 @@
  #define ID_KEY "id"
  #define FIRST_NAME_KEY "fn"
  #define LAST_NAME_KEY "ln"
+ #define ACTIVITY_KEY "actv"
  #define CITY_KEY "cin"
  #define CITY_ID_KEY "cii"
  #define COUNTRY_KEY "con"
@@ -63,11 +64,11 @@
 #define TYPE_SELECT "select"
 #define TYPE_ARRAY "array"
 
-int parser_profile(xmlDocPtr doc, char *json, char* remixmid);
+int parser_profile(xmlDocPtr doc, char *json, char* remixmid, gboolean image);
 int parser_image(xmlDocPtr doc, char* data, char* url);
-int parser_friends(char* json, xmlDocPtr doc);
+int parser_friends(char* json, xmlDocPtr doc, int counter);
  
-int parser_messages(char *json, xmlDocPtr doc, char *message_id);
+int parser_messages(char *json, xmlDocPtr doc, gchar *message_id, int counter, int max_count);
 
 /**
  * Parses json data, form xml
